/**
 * This software was developed at the National Institute of Standards and
 * Technology (NIST) by employees of the Federal Government in the course
 * of their official duties. Pursuant to title 17 Section 105 of the
 * United States Code, this software is not subject to copyright protection
 * and is in the public domain. NIST assumes no responsibility  whatsoever for
 * its use by other parties, and makes no guarantees, expressed or implied,
 * about its quality, reliability, or any other characteristic.
 **/
#include <cstdlib>
#include <iostream>
#include <limits>
#include <iomanip>
#include <fstream>
#include <cmath>

#include "read_jpeg.h"
#include "jpeglib.h"
#include <string.h>   // for memcpy

using namespace std;

// Reads a PGM file into an iris_sample object.
// This function isn't intended to fully support the PGM format, only enough
// to read the validation images.
bool read_jpeg(const string& jpeg_file, IREX9::iris_sample& is)
{
   FILE* infile;

   if ((infile = fopen(jpeg_file.c_str(), "rb")) == NULL)
   {
      cerr << "Cannot open the input file " << jpeg_file << endl;
      return false;
   }

   struct jpeg_error_mgr jerr;
   struct jpeg_decompress_struct cinfo;

   cinfo.err = jpeg_std_error(&jerr);
   jpeg_create_decompress(&cinfo);

   // Specify data source (file).
   jpeg_stdio_src(&cinfo, infile);

   // Read file parameters from header.
   if (jpeg_read_header(&cinfo, TRUE) != JPEG_HEADER_OK)
   {
      cerr << "Error reading header for " << jpeg_file << endl;
      return false;
   }

   jpeg_start_decompress(&cinfo);

   // Row width for buffer
   const int row_stride = cinfo.output_width * cinfo.output_components;

   // Allocate space for decompressed image.
   const int num_bytes = row_stride * cinfo.output_height;
   is.data = new uint8_t[num_bytes];

   uint8_t* offset = is.data;

   // Make a one-row-high sample array that will go away when done with image.
   JSAMPARRAY buffer = (*cinfo.mem->alloc_sarray)((j_common_ptr)&cinfo, JPOOL_IMAGE, row_stride, 1);

   while (cinfo.output_scanline < cinfo.output_height)
   {
      // Read scanlines one row at a time
      jpeg_read_scanlines(&cinfo, buffer, 1);

      // Copy result to output buffer.
      memcpy(offset, buffer[0], row_stride);
      offset += row_stride;
   }

   jpeg_finish_decompress(&cinfo);
   jpeg_destroy_decompress(&cinfo);

   fclose(infile);

   // Populate other fields in iris_sample.
   is.image_width = cinfo.output_width;
   is.image_height = cinfo.output_height;
   is.bit_depth = cinfo.output_components * 8;
   is.wavelength = 0;                             // indicating unknown
   is.eye = IREX9::UNDEFINED;
   is.boundary = NULL;

   return true;
}

